searchNodes=[{"doc":"Dataloader Dataloader provides an easy way efficiently load data in batches. It's inspired by https://github.com/facebook/dataloader, although it makes some small API changes to better suit Elixir use cases. Central to Dataloader is the idea of a source. A single Dataloader struct can have many different sources, which represent different ways to load data. Here's an example of a data loader using an ecto source, and then loading some organization data. source = Dataloader.Ecto . new ( MyApp.Repo ) # setup the loader loader = Dataloader . new |&gt; Dataloader . add_source ( :db , source ) # load some things loader = loader |&gt; Dataloader . load ( :db , Organization , 1 ) |&gt; Dataloader . load_many ( :db , Organization , [ 4 , 9 ] ) # actually retrieve them loader = Dataloader . run ( loader ) # Now we can get whatever values out we want organizations = Dataloader . get_many ( loader , :db , Organization , [ 1 , 4 ] ) This will do a single SQL query to get all organizations by ids 1,4, and 9. You can load multiple batches from multiple sources, and then when run/1 is called batch will be loaded concurrently. Here we named the source :db within our dataloader. More commonly though if you're using Phoenix you'll want to name it after one of your contexts, and have a different source used for each context. This provides an easy way to enforce data access rules within each context. See the Dataloader.Ecto moduledocs for more details Options There are two configuration options: timeout - The maximum timeout to wait for running a source, defaults to 1s more than the maximum timeout of all added sources. Set with care, timeouts should really only be set on sources. get_policy - This configures how the dataloader will behave when fetching data which may have errored when we tried to load it. These can be set as part of the new/1 call. So, for example, to configure a dataloader that returns nil on error with a 5s timeout: loader = Dataloader . new ( get_policy : :return_nil_on_error , timeout : :timer . seconds ( 5 ) ) get_policy There are three implemented behaviours for this: raise_on_error (default)- If successful, calling get/4 or get_many/4 will return the value. If there was an exception when trying to load any of the data, it will raise that exception return_nil_on_error - Behaves similar to raise_on_error but will just return nil instead of raising . It will still log errors tuples - This will return {:ok, value} / {:error, reason} tuples depending on a successful or failed load, allowing for more fine-grained error handling if required","ref":"Dataloader.html","title":"Dataloader","type":"module"},{"doc":"","ref":"Dataloader.html#add_source/3","title":"Dataloader.add_source/3","type":"function"},{"doc":"This is a helper method to run a set of async tasks in a separate supervision tree which: Is run by a supervisor linked to the main process. This ensures any async tasks will get killed if the main process is killed. Spawns a separate task which traps exits for running the provided function. This ensures we will always have some output, but are not setting :trap_exit on the main process. NOTE : The provided fun must accept a Task.Supervisor as its first argument, as this function will prepend the relevant supervisor to args See run_task/3 for an example of a fun implementation, this will return whatever that returns.","ref":"Dataloader.html#async_safely/4","title":"Dataloader.async_safely/4","type":"function"},{"doc":"","ref":"Dataloader.html#default_timeout/0","title":"Dataloader.default_timeout/0","type":"function"},{"doc":"","ref":"Dataloader.html#get/4","title":"Dataloader.get/4","type":"function"},{"doc":"","ref":"Dataloader.html#get_many/4","title":"Dataloader.get_many/4","type":"function"},{"doc":"","ref":"Dataloader.html#load/4","title":"Dataloader.load/4","type":"function"},{"doc":"","ref":"Dataloader.html#load_many/4","title":"Dataloader.load_many/4","type":"function"},{"doc":"Create a new Dataloader instance. See moduledoc for available options","ref":"Dataloader.html#new/1","title":"Dataloader.new/1","type":"function"},{"doc":"","ref":"Dataloader.html#pending_batches?/1","title":"Dataloader.pending_batches?/1","type":"function"},{"doc":"This used to be used by both the Dataloader module for running multiple source queries concurrently, and the KV and Ecto sources to actually run separate batch fetches (e.g. for Posts and Users at the same time). The problem was that the behaviour between the sources and the parent Dataloader was actually slightly different. The Dataloader -specific behaviour has been pulled out into run_tasks/4 Please use async_safely/3 instead of this for fetching data from sources","ref":"Dataloader.html#pmap/3","title":"Dataloader.pmap/3","type":"function"},{"doc":"","ref":"Dataloader.html#put/5","title":"Dataloader.put/5","type":"function"},{"doc":"","ref":"Dataloader.html#run/1","title":"Dataloader.run/1","type":"function"},{"doc":"This helper function will call fun on all items asynchronously, returning a map of :ok / :error tuples, keyed off the items . For example: iex&gt; Dataloader . run_tasks ( [ 1 , 2 , 3 ] , fn x -&gt; x * x end , [ ] ) %{ 1 =&gt; { :ok , 1 } , 2 =&gt; { :ok , 4 } , 3 =&gt; { :ok , 9 } } Similarly, for errors: iex&gt; Dataloader . run_tasks ( [ 1 , 2 , 3 ] , fn _x -&gt; Process . sleep ( 5 ) end , [ timeout : 1 ] ) %{ 1 =&gt; { :error , :timeout } , 2 =&gt; { :error , :timeout } , 3 =&gt; { :error , :timeout } } By default, tasks are run asynchronously. To run them synchronously, provide async?: false .","ref":"Dataloader.html#run_tasks/3","title":"Dataloader.run_tasks/3","type":"function"},{"doc":"","ref":"Dataloader.html#t:option/0","title":"Dataloader.option/0","type":"type"},{"doc":"","ref":"Dataloader.html#t:source_name/0","title":"Dataloader.source_name/0","type":"type"},{"doc":"","ref":"Dataloader.html#t:t/0","title":"Dataloader.t/0","type":"type"},{"doc":"Ecto source for Dataloader This defines a schema and an implementation of the Dataloader.Source protocol for handling Ecto related batching. A simple Ecto source only needs to know about your application's Repo. Basic Usage Querying by primary key (analogous to Ecto.Repo.get/3): source = Dataloader.Ecto . new ( MyApp.Repo ) loader = Dataloader . new |&gt; Dataloader . add_source ( Accounts , source ) |&gt; Dataloader . load ( Accounts , User , 1 ) |&gt; Dataloader . load_many ( Accounts , Organization , [ 4 , 9 ] ) |&gt; Dataloader . run organizations = Dataloader . get_many ( loader , Accounts , Organization , [ 4 , 9 ] ) Querying for associations. Here we look up the :users association on all the organizations, and the :organization for a single user. loader = loader |&gt; Dataloader . load ( Accounts , :organization , user ) |&gt; Dataloader . load_many ( Accounts , :users , organizations ) |&gt; Dataloader . run Querying by a column other than the primary key: loader = loader |&gt; Dataloader . load ( Accounts , { :one , User } , name : &quot;admin&quot; ) |&gt; Dataloader . run Here we pass a keyword list of length one. It is only possible to query by one column here; for more complex queries, see &quot;filtering&quot; below. Notice here that we need to also specify the cardinality in the batch_key ( :many or :one ), which will decide whether to return a list or a single value (or nil). This is because the column may not be a key and there may be multiple matching records. Note also that even if we are returning :many values here from multiple matching records, this is still a call to Dataloader.load/4 rather than Dataloader.load_many/4 because there is only one val specified. Filtering / Ordering Dataloader.Ecto.new/2 can receive a 2 arity function that can be used to apply broad ordering and filtering rules, as well as handle parameters source = Dataloader.Ecto . new ( MyApp.Repo , query : &amp; Accounts . query / 2 ) loader = Dataloader . new |&gt; Dataloader . add_source ( Accounts , source ) When we call Dataloader.load/4 we can pass in a tuple as the batch key with a keyword list of parameters in addition to the queryable or assoc_field # with a queryable loader |&gt; Dataloader . load ( Accounts , { User , order : :name } , 1 ) # or an association loader |&gt; Dataloader . load_many ( Accounts , { :users , order : :name } , organizations ) # this is still supported loader |&gt; Dataloader . load ( Accounts , User , 1 ) # as is this loader |&gt; Dataloader . load ( :accounts , :user , organization ) In all cases the Accounts.query function would be: def query ( User , params ) do field = params [ :order ] || :id from u in User , order_by : [ asc : field ( u , ^ field ) ] end def query ( queryable , _ ) do queryable end If we query something that ends up using the User schema, whether directly or via association, the query/2 function will match on the first clause and we can handle the params. If no params are supplied, the params arg defaults to source.default_params which itself defaults to %{} . default_params is an extremely useful place to store values like the current user: source = Dataloader.Ecto . new ( MyApp.Repo , [ query : &amp; Accounts . query / 2 , default_params : %{ current_user : current_user } , ] ) loader = Dataloader . new |&gt; Dataloader . add_source ( Accounts , source ) |&gt; Dataloader . load_many ( Accounts , Organization , ids ) |&gt; Dataloader . run # the query function def query ( Organization , %{ current_user : user } ) do from o in Organization , join : m in assoc ( o , :memberships ) , where : m . user_id == ^ user . id end def query ( queryable , _ ) do queryable end In our query function we are pattern matching on the current user to make sure that we are only able to lookup data in organizations that the user actually has a membership in. Additional options you specify IE {Organization, %{order: :asc}} are merged into the default. Custom batch queries There are cases where you want to run the batch function yourself. To do this we can add a custom run_batch/5 callback to our source. The run_batch/5 function is executed with the query returned from the query/2 function. For example, we want to get the post count for a set of users. First we add a custom run_batch/5 function. def run_batch ( _ , query , :post_count , users , repo_opts ) do user_ids = Enum . map ( users , &amp; &amp;1 . id ) default_count = 0 result = query |&gt; where ( [ p ] , p . user_id in ^ user_ids ) |&gt; group_by ( [ p ] , p . user_id ) |&gt; select ( [ p ] , { p . user_id , count ( &quot;*&quot; ) } ) |&gt; Repo . all ( repo_opts ) |&gt; Map . new ( ) for %{ id : id } &lt;- users do [ Map . get ( result , id , default_count ) ] end end # Fallback to original run_batch def run_batch ( queryable , query , col , inputs , repo_opts ) do Dataloader.Ecto . run_batch ( Repo , queryable , query , col , inputs , repo_opts ) end This function is supplied with a list of users, does a query and will return the post count for each of user. If the user id is not found in the resultset, because the user has no posts, we return a post count of 0. Now we need to call run_batch/5 from dataloader. First we add a few posts to the database. After that, the custom run_batch/5 function is provided to the Dataloader source. Now, we can load the post count for several users. When the dataloader runs it will call the custom run_batch/5 and we can retrieve the posts counts for each individual user. [ user1 , user2 ] = [ % User { id : 1 } , % User { id : 2 } ] rows = [ %{ user_id : user1 . id , title : &quot;foo&quot; , published : true } , %{ user_id : user1 . id , title : &quot;baz&quot; , published : false } ] _ = Repo . insert_all ( Post , rows ) source = Dataloader.Ecto . new ( Repo , query : &amp; query / 2 , run_batch : &amp; run_batch / 5 ) loader = Dataloader . new ( ) |&gt; Dataloader . add_source ( Posts , source ) loader = loader |&gt; Dataloader . load ( Posts , { :one , Post } , post_count : user1 ) |&gt; Dataloader . load ( Posts , { :one , Post } , post_count : user2 ) |&gt; Dataloader . run ( ) # Returns 2 Dataloader . get ( loader , Posts , { :one , Post } , post_count : user1 ) # Returns 0 Dataloader . get ( loader , Posts , { :one , Post } , post_count : user2 ) Additional params for the query/2 function can be passed to the load functions with a 3-tuple. For example, to limit the above example to only return published we can add a query function to filter the published posts: def query ( Post , %{ published : published } ) do from p in Post , where : p . published == ^ published end def query ( queryable , _ ) do queryable end And we can return the published posts with a 3-tuple on the loader: loader = loader |&gt; Dataloader . load ( Posts , { :one , Post } , post_count : user1 ) |&gt; Dataloader . load ( Posts , { :one , Post , %{ published : true } } , post_count : user1 ) |&gt; Dataloader . run ( ) # Returns 2 Dataloader . get ( loader , Posts , { :one , Post } , post_count : user1 ) # Returns 1 Dataloader . get ( loader , Posts , { :one , Post , %{ published : true } } , post_count : user1 )","ref":"Dataloader.Ecto.html","title":"Dataloader.Ecto","type":"module"},{"doc":"Create an Ecto Dataloader source. This module handles retrieving data from Ecto for dataloader. It requires a valid Ecto Repo. It also accepts a repo_opts: option which is handy for applying options to any calls to Repo functions that this module makes. For example, you can use this module in a multi-tenant context by using the prefix option: Dataloader.Ecto . new ( MyApp.Repo , repo_opts : [ prefix : &quot;tenant&quot; ] )","ref":"Dataloader.Ecto.html#new/2","title":"Dataloader.Ecto.new/2","type":"function"},{"doc":"Default implementation for loading a batch. Handles looking up records by column","ref":"Dataloader.Ecto.html#run_batch/6","title":"Dataloader.Ecto.run_batch/6","type":"function"},{"doc":"","ref":"Dataloader.Ecto.html#t:batch_fun/0","title":"Dataloader.Ecto.batch_fun/0","type":"type"},{"doc":"","ref":"Dataloader.Ecto.html#t:opt/0","title":"Dataloader.Ecto.opt/0","type":"type"},{"doc":"","ref":"Dataloader.Ecto.html#t:query_fun/0","title":"Dataloader.Ecto.query_fun/0","type":"type"},{"doc":"","ref":"Dataloader.Ecto.html#t:repo_opts/0","title":"Dataloader.Ecto.repo_opts/0","type":"type"},{"doc":"","ref":"Dataloader.Ecto.html#t:t/0","title":"Dataloader.Ecto.t/0","type":"type"},{"doc":"Simple KV based Dataloader source. This module is a simple key value based data loader source. You must supply a function that accepts ids, and returns a map of values keyed by id. Example def datasource do Dataloader.KV . new ( &amp; query / 2 , max_concurrency : 1 ) end def query ( :comments , posts ) do Map . new ( posts , fn %{ id : post_id } = post -&gt; { post , Comments . find_by ( post_id : post_id ) } end ) end","ref":"Dataloader.KV.html","title":"Dataloader.KV","type":"module"},{"doc":"Create a KV Dataloader source. Dataloader runs tasks concurrently using Task.async_stream/3 . The concurrency of a KV Dataloader source and the time tasks are allowed to run can be controlled via options (see the &quot;Options&quot; section below). Options :max_concurrency - sets the maximum number of tasks to run at the same time. Defaults to twice the number of schedulers online (see System.schedulers_online/0 ). :timeout - the maximum amount of time (in milliseconds) a task is allowed to execute for. Defaults to 30000 . :async? - set to false to disable the asynchronous behavior mentioned above.","ref":"Dataloader.KV.html#new/2","title":"Dataloader.KV.new/2","type":"function"},{"doc":"","ref":"Dataloader.Source.html","title":"Dataloader.Source","type":"protocol"},{"doc":"Fetch the result found under the given batch and item keys.","ref":"Dataloader.Source.html#fetch/3","title":"Dataloader.Source.fetch/3","type":"function"},{"doc":"Enqueue an item to be loaded under a given batch","ref":"Dataloader.Source.html#load/3","title":"Dataloader.Source.load/3","type":"function"},{"doc":"Determine if there are any batches that have not yet been run.","ref":"Dataloader.Source.html#pending_batches?/1","title":"Dataloader.Source.pending_batches?/1","type":"function"},{"doc":"Put a value into the results. Useful for warming caches. The source is permitted to reject the value.","ref":"Dataloader.Source.html#put/4","title":"Dataloader.Source.put/4","type":"function"},{"doc":"Run any batches queued up for this source.","ref":"Dataloader.Source.html#run/1","title":"Dataloader.Source.run/1","type":"function"},{"doc":"Returns the timeout (in ms) for the source. This is important for ensuring the dataloader obeys the timeouts when running multiple sources concurrently","ref":"Dataloader.Source.html#timeout/1","title":"Dataloader.Source.timeout/1","type":"function"},{"doc":"","ref":"Dataloader.Source.html#t:batch_key/0","title":"Dataloader.Source.batch_key/0","type":"type"},{"doc":"","ref":"Dataloader.Source.html#t:item_key/0","title":"Dataloader.Source.item_key/0","type":"type"},{"doc":"","ref":"Dataloader.Source.html#t:t/0","title":"Dataloader.Source.t/0","type":"type"},{"doc":"","ref":"Dataloader.GetError.html","title":"Dataloader.GetError","type":"exception"},{"doc":"Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Feature: Handle sync vs async via an option passed in at Dataloader.new() and KV.new(). Example: def dataloader ( ctx \\\\ %{ } ) do Dataloader . new ( async? : ! MyApp.Repo . in_transaction? ( ) ) |&gt; Dataloader . add_source ( MainLoader , MainLoader . data ( ctx ) ) end Note that ecto sources will automatically handle whether to perform queries sync or async depending on if it's in a transaction. IMPORTANT: The option MUST be passed to Dataloader.new for this to work properly. In order for this to happen automatically, upgrade to 2.0","ref":"changelog.html#v1-0-11-unreleased","title":"Changelog - v1.0.11 (unreleased)","type":"extras"},{"doc":"Support Telemetry 1.0 Various typo and doc fixes.","ref":"changelog.html#v1-0-10-2021-12-22","title":"Changelog - v1.0.10 2021-12-22","type":"extras"},{"doc":"Improvement: KV data sources now pass opts to Dataloader.run_tasks/3 ( #94 ) Improvement: Add support for where and join_where in has_many and many_to_many associations ( #119 ) Bug Fix: Avoid preloads in lateral subqueries ( #98 ) Bug Fix: Use existing database prefix when preloading via lateral join ( #107 ) Bug Fix: Fix issue with Dataloader.Ecto.run_batches/1 ( #126 ) Doc improvements","ref":"changelog.html#v1-0-9-2021-08-20","title":"Changelog - v1.0.9 2021-08-20","type":"extras"},{"doc":"Improvement: Telemetry integration ( #89 ) Improvement: limit support via lateral joins ( #93 )","ref":"changelog.html#v1-0-8-2020-08-25","title":"Changelog - v1.0.8 - 2020-08-25","type":"extras"},{"doc":"Improvement: Optimize out inspect calls when keys are not found ( #84 ) KV source bug fixes ( #75 ) Doc improvements","ref":"changelog.html#v1-0-7-2020-01-09","title":"Changelog - v1.0.7 - 2020-01-09","type":"extras"},{"doc":"Bug Fix: Depend on ecto instead of ecto_sql . Fixes Ecto 2.x support ( #67 )","ref":"changelog.html#v1-0-6-2019-02-06","title":"Changelog - v1.0.6 - 2019-02-06","type":"extras"},{"doc":"Improvement: Ecto 3 support (which is just improving the tests) Improvement: Better error messages if you pass a non-ecto schema to Dataloader.load/4 or Datalaoder.get/4 Bug Fix: Fix dialyzer spec for run_batch function Bug Fix: Fix loading of nested through associations ( #65 ) Breaking change: Dependency changed from ecto to ecto_sql which unintentionally breaks Dataloader.Ecto on Ecto 2.x projects ( #67 ).","ref":"changelog.html#v1-0-5-2019-02-03","title":"Changelog - v1.0.5 - 2019-02-03","type":"extras"},{"doc":"Bug Fix: Poor supervisor structure has been improved, which fixes large memory usage issues","ref":"changelog.html#v1-0-4-2018-09-14","title":"Changelog - v1.0.4 - 2018-09-14","type":"extras"},{"doc":"Enhancement: Improved error handling when sources fail to load This provides two additional configurable methods of error handling (these options can be passed to Dataloader.new/1 ): :return_nil_on_error - This is the previous default. Errors are logged, and values return nil . :raise_on_error - this will raise a Dataloader.GetError when one of the get methods are called. This is now the default behaviour for handling errors :tuples - this changes the get/4 / get_many/4 methods to return :ok / :error tuples instead of just the value. This frees up the caller to handle errors any way they see fit Enhancement: Improved caching characteristics on the KV source Enhancement: More flexible cardinality mapping for Ecto source Enhancement: Uniq the batched KV and Ecto values Bug Fix: When using the Ecto source it properly coerces all inputs for known fields. Note: when upgrading it may be necessary to clear out your _build directory so that the compiler picks up the protocol change. Breaking Changes Errors are raised instead of silently returning nil. See :return_nil_on_error option to keep the previous behavior. Dataloader.get/4 will raise an error if you pass in an item_key that was not previously loaded. See https://github.com/absinthe-graphql/dataloader/issues/52 for discussion.","ref":"changelog.html#v1-0-3-2018-08-19","title":"Changelog - v1.0.3 - 2018-08-19","type":"extras"},{"doc":"Enhancement: Custom batch functions for Ecto","ref":"changelog.html#v1-0-2-2018-04-10","title":"Changelog - v1.0.2 - 2018-04-10","type":"extras"},{"doc":"Bug Fix: Ecto source properly caches results now.","ref":"changelog.html#v1-0-1-2018-02-04","title":"Changelog - v1.0.1 - 2018-02-04","type":"extras"},{"doc":"Initial release","ref":"changelog.html#v1-0-0-2017-11-13","title":"Changelog - v1.0.0 - 2017-11-13","type":"extras"},{"doc":"Dataloader Dataloader provides an easy way efficiently load data in batches. It's inspired by https://github.com/facebook/dataloader , although it makes some small API changes to better suit Elixir use cases.","ref":"readme.html","title":"Dataloader","type":"extras"},{"doc":"The package can be installed by adding :dataloader to your list of dependencies in mix.exs : def deps do [ { :dataloader , &quot;~&gt; 1.0.0&quot; } ] end Note: Dataloader requires Elixir 1.10 or higher.","ref":"readme.html#installation","title":"Dataloader - Installation","type":"extras"},{"doc":"See CHANGELOG for upgrade steps between versions.","ref":"readme.html#upgrading","title":"Dataloader - Upgrading","type":"extras"},{"doc":"Dataloader hexdocs . For the tutorial, guides, and general information about Absinthe-related projects, see http://absinthe-graphql.org .","ref":"readme.html#documentation","title":"Dataloader - Documentation","type":"extras"},{"doc":"Central to Dataloader is the idea of a source. A single Dataloader struct can have many different sources, which represent different ways to load data. Here's an example of a data loader using an Ecto source, and then loading some organization data. source = Dataloader.Ecto . new ( MyApp.Repo ) # setup the loader loader = Dataloader . new |&gt; Dataloader . add_source ( :db , source ) # load some things loader = loader |&gt; Dataloader . load ( :db , Organization , 1 ) |&gt; Dataloader . load_many ( :db , Organization , [ 4 , 9 ] ) # actually retrieve them loader = Dataloader . run ( loader ) # Now we can get whatever values out we want organizations = Dataloader . get_many ( loader , :db , Organization , [ 1 , 4 ] ) This will do a single SQL query to get all organizations by ids 1, 4, and 9. You can load multiple batches from multiple sources, and then when run/1 is called batch will be loaded concurrently. Here we named the source :db within our dataloader. More commonly though if you're using Phoenix you'll want to name it after one of your contexts, and have a different source used for each context. This provides an easy way to enforce data access rules within each context. See the Dataloader.Ecto moduledocs for more details Sources Dataloader ships with two different built in sources. The first is the Ecto source for easily pulling out data with ecto. The other is a simple KV key value source. See each module for its respective documentation. Anything that implements the Dataloader.Source protocol can act as a source.","ref":"readme.html#usage","title":"Dataloader - Usage","type":"extras"},{"doc":"The project is under constant improvement by a growing list of contributors, and your feedback is important. Please join us in Slack ( #absinthe-graphql under the Elixir Slack account) or the Elixir Forum (tagged absinthe ). Please remember that all interactions in our official spaces follow our Code of Conduct .","ref":"readme.html#community","title":"Dataloader - Community","type":"extras"},{"doc":"See the GitHub organization .","ref":"readme.html#related-projects","title":"Dataloader - Related Projects","type":"extras"},{"doc":"Please follow contribution guide .","ref":"readme.html#contributing","title":"Dataloader - Contributing","type":"extras"},{"doc":"See LICENSE.md .","ref":"readme.html#license","title":"Dataloader - License","type":"extras"},{"doc":"Telemetry Dataloader uses telemetry to instrument its activity. Call :telemetry.attach/4 or :telemetry.attach_many/4 to attach your handler function to any of the following event names: [:dataloader, :source, :run, :start] when the dataloader processing starts [:dataloader, :source, :run, :stop] when the dataloader processing finishes [:dataloader, :source, :batch, :run, :start] when the dataloader starts processing a single batch [:dataloader, :source, :batch, :run, :stop] when the dataloader finishes processing a single batch Telemetry handlers are called with measurements and metadata . For details on what is passed, checkout Dataloader .","ref":"telemetry.html","title":"Telemetry","type":"extras"},{"doc":"As an example, you could attach a handler in an iex -S mix shell. Paste in: :telemetry . attach_many ( :demo , [ [ :dataloader , :source , :run , :stop ] ] , fn event_name , measurements , metadata , _config -&gt; %{ event_name : event_name , measurements : measurements , metadata : metadata } |&gt; IO . inspect ( ) end , [ ] ) After a query is executed, you'll see something like: %{ event_name : [ :dataloader , :source , :run , :stop ] , measurements : %{ duration : 112151 } , metadata : %{ dataloader : % Dataloader { options : [ get_policy : :raise_on_error ] , sources : ... } , id : - 576460752303420441 } }","ref":"telemetry.html#interactive-telemetry","title":"Telemetry - Interactive Telemetry","type":"extras"}]